/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.jmx;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.util.List;

import org.junit.Test;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.EndpointDirectoryAdministration;
import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.EndpointDirectoryAdministrationException;
import org.ow2.petals.admin.api.exception.EndpointDirectoryRegexpPatternException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.endpoint.Endpoint;
import org.ow2.petals.admin.endpoint.EndpointDirectoryView;
import org.ow2.petals.jmx.api.mock.EndpointDirectoryClientMock;

/**
 * Unit tests of {@link EndpointDirectoryAdministration}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class JMXEndpointDirectoryAdministrationTest extends JMXAbstractAdministrationTest {

    /**
     * Check the {@link JMXEndpointDirectoryAdministration#getEndpointDirectoryContent(String, String, String, String)}
     * against a filtering where:
     * <ul>
     * <li>the endpoint directory is composed of:
     * <ul>
     * <li>Endpoints:
     * <ul>
     * <li>HelloEndpoint-TargetContainer0, deployed on container 'target-container-0'</li>
     * <li>HelloEndpoint-TargetContainer1, deployed on container 'target-container-1'</li>
     * </ul>
     * </li>
     * <li>Services:
     * <ul>
     * <li>{http://petals.ow2.org/service/}HelloService0 implemented by the endpoint 'HelloEndpoint-TargetContainer0'
     * </li>
     * <li>{http://petals.ow2.org/service/}HelloService1 implemented by the endpoint 'HelloEndpoint-TargetContainer1'
     * </li>
     * </ul>
     * </li>
     * <li>Interfaces:
     * <ul>
     * <li>{http://petals.ow2.org/}HelloPortType implemented by services '{http://petals.ow2.org/}HelloService0' and
     * '{http://petals.ow2.org/}HelloService1'</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>when filtering on container name with the regexp 'target-container-.*':
     * <ul>
     * <li>no error,</li>
     * <li>The endpoints 'HelloEndpoint-TargetContainer0' and 'HelloEndpoint-TargetContainer1' are returned</li>
     * </ul>
     * </li>
     * <li>when filtering on container name with the regexp 'target-container-1':
     * <ul>
     * <li>no error,</li>
     * <li>only the endpoint 'HelloEndpoint-TargetContainer1' is returned</li>
     * </ul>
     * </li>
     * <li>when filtering on container name with the regexp 'unknown-endpoint':
     * <ul>
     * <li>no error,</li>
     * <li>no endpoint is returned</li>
     * </ul>
     * </li>
     * <li>when filtering on container name with the regexp 'target-container-.*' and filtering on service
     * '{http://petals.ow2.org/service/}HelloService0':
     * <ul>
     * <li>no error,</li>
     * <li>only the endpoint 'HelloEndpoint-TargetContainer0' is returned</li>
     * </ul>
     * </ul>
     * </p>
     */
    @Test
    public void testGetEndpointDirectoryContent()
            throws DuplicatedServiceException, MissingServiceException, EndpointDirectoryRegexpPatternException,
            EndpointDirectoryAdministrationException, ContainerAdministrationException {

        final PetalsAdministration adminApi = PetalsAdministrationFactory.getInstance().newPetalsAdministrationAPI();
        final ContainerAdministration containerAdm = adminApi.newContainerAdministration();
        final EndpointDirectoryAdministration endpointDirectoryAdm = adminApi.newEndpointDirectoryAdministration();

        containerAdm.connect("localhost", 7700, "petals", "petals");

        // Check against container name regexp: 'target-container-.*'
        {
            final EndpointDirectoryView endpointDirectoryView = endpointDirectoryAdm
                    .getEndpointDirectoryContent("target-container-.*", null, null, null);
            assertNotNull(endpointDirectoryView);
            final List<Endpoint> endpoints = endpointDirectoryView.getAllEndpoints();
            assertNotNull(endpoints);
            assertEquals("Unexpected number of endpoints", 2, endpoints.size());
            Endpoint edp0 = null;
            Endpoint edp1 = null;
            for (final Endpoint edp : endpoints) {
                if (EndpointDirectoryClientMock.EDP_CONT0_SVC0.equals(edp.getEndpointName())) {
                    edp0 = edp;
                } else if (EndpointDirectoryClientMock.EDP_CONT1_SVC1.equals(edp.getEndpointName())) {
                    edp1 = edp;
                } else {
                    fail("Unexpected endpoint found: " + edp.getEndpointName());
                }
            }
            assertNotNull("Endpoint #0 is missing", edp0);
            assertNotNull("Endpoint #1 is missing", edp1);
        }

        // Check against container name regexp: 'target-container-1'
        {
            final EndpointDirectoryView endpointDirectoryView = endpointDirectoryAdm
                    .getEndpointDirectoryContent("target-container-1", null, null, null);
            assertNotNull(endpointDirectoryView);
            final List<Endpoint> endpoints = endpointDirectoryView.getAllEndpoints();
            assertNotNull(endpoints);
            assertEquals("Unexpected number of endpoints", 1, endpoints.size());
            assertEquals("Unexpected container name", EndpointDirectoryClientMock.CONTAINER1,
                    endpoints.get(0).getContainerName());
            assertEquals("Unexpected endpoint name", EndpointDirectoryClientMock.EDP_CONT1_SVC1,
                    endpoints.get(0).getEndpointName());
        }

        // Check against container name regexp: 'unknown-endpoint'
        {
            final EndpointDirectoryView endpointDirectoryView = endpointDirectoryAdm
                    .getEndpointDirectoryContent("unknown-endpoint", null, null, null);
            assertNotNull(endpointDirectoryView);
            final List<Endpoint> endpoints = endpointDirectoryView.getAllEndpoints();
            assertNotNull(endpoints);
            assertEquals("Unexpected number of endpoints", 0, endpoints.size());
        }

        // Check against container name regexp 'target-container-.*' and service name regexp
        // '{http://petals.ow2.org/service/}HelloService0'
        {
            final EndpointDirectoryView endpointDirectoryView = endpointDirectoryAdm.getEndpointDirectoryContent(
                    "target-container-.*", null, "\\{http://petals.ow2.org/service/\\}HelloService0", null);
            assertNotNull(endpointDirectoryView);
            final List<Endpoint> endpoints = endpointDirectoryView.getAllEndpoints();
            assertNotNull(endpoints);
            assertEquals("Unexpected number of endpoints", 1, endpoints.size());
            assertEquals("Unexpected container name", EndpointDirectoryClientMock.CONTAINER0,
                    endpoints.get(0).getContainerName());
            assertEquals("Unexpected endpoint name", EndpointDirectoryClientMock.EDP_CONT0_SVC0,
                    endpoints.get(0).getEndpointName());
            assertEquals("Unexpected service name", EndpointDirectoryClientMock.SVC0.toString(),
                    endpoints.get(0).getServiceName());
        }
    }

}
